package eu.simstadt.nf4j;

/**
 * This exception may be thrown by classes of the nf4j package if/on ...
 * 
 * - the connector is null
 * - malformed URLs
 * - missing or malformed XML reports
 * - HTTP failures
 * 
 * @author Marcel Bruse
 */
public class FailedTransmissionException extends Exception {

	private static final long serialVersionUID = -3530932388888249528L;

	/** An textual description of the error. */
	private String message;
	
	/** Standard constructor. */
	public FailedTransmissionException() {}
	
	/**
	 * Constructor with error message and without nested cause.
	 * 
	 * @param message The error message.
	 */
	public FailedTransmissionException(String message) {
		this(message, null);
	}
	
	/**
	 * Constructor with cause and without error message.
	 * 
	 * @param cause The nested cause of this exception.
	 */
	public FailedTransmissionException(Throwable cause) {
		this(null, cause);
	}
	
	/**
	 * Constructor with error message and nested cause.
	 * 
	 * @param message The error message.
	 * @param cause The nested cause of this exception.
	 */
	public FailedTransmissionException(String message, Throwable cause) {
		this.message = message;		
		initCause(cause);
	}
	
	/**
	 * @return Returns the error message, if present.
	 */
	@Override
	public String getMessage() {
		return message;
	}
	
}
