package eu.simstadt.nf4j;
/**
* An import job is a proxy object for an actual nF import job. Import jobs are used to store data in your nF database. 
* Every import job has to have a valid job descriptor and/or a job id. 
* 
* @author Marcel Bruse
* 
* @param <D> The descriptor type for the import job implementation.
*/
public abstract class ImportJob<D extends ImportJobDescriptor> extends Job {

	/** Every job should have a (valid) job descriptor. */
	protected D descriptor;
	
	/**
	 * This constructor forces the job to have a description and a connector instance.
	 * 
	 * @param connector The job will use this connector to synchronize itself with the nF.
	 * @param descriptor The description of this job.
	 */
	public ImportJob(D descriptor, Connector<?, ?> connector) {
		this.descriptor = descriptor;
		this.connector = connector;
	}
	
	/**
	 * This constructor forces the job to have a id and a connector instance.
	 * 
	 * @param id The job id. If you call updateStatus() and the nF "knows" the job id, then the job status
	 * will be updated. If you call updateStatus() and the job id is "unkown" on the server side, then     
	 * @param connector The job will use this connector to synchronize itself with the nF.
	 */
	public ImportJob(int id, Connector<?, ?> connector) {
		this.id = id;
		this.connector = connector;
	}
	
	/**
	 * @return Returns the description of this job.
	 */
	public D getDescriptor() {
		return descriptor;
	}
	
}
