package eu.simstadt.nf4j.async;

import java.io.File;
import java.util.Optional;

import eu.simstadt.nf4j.FailedTransmissionException;
import eu.simstadt.nf4j.JobStatus;

/**
 * This task downloads export job results asynchronously within its jobs separate download thread.
 * You can cancel this task by calling job.cancel().
 *  
 * @author Marcel Bruse
 */
public class DownloadTask implements Runnable {

	/** The finished export job for which you want to download a CityGML result. */
	private AsyncExportJob job;

	/**
	 * Constructor with finished(!) asynchronous export job.
	 *  
	 * @param job The export job for which you want to download the CityGML result.
	 */
	public DownloadTask(AsyncExportJob job) {
		this.job = job;
	}
	
	/**
	 * This method performs the download operation asynchronously in the export jobs download thread.
	 * Job status listeners will be notified upon the finished download.
	 */
	@Override
	public void run() {
		HTTPConnection connector = (HTTPConnection) job.getConnector();
		try {
			File file = connector.requestExportJobResult(job);
			job.setResult(file);
			job.setStatus(JobStatus.DOWNLOAD, Optional.empty());
		} catch (FailedTransmissionException ex) {
			// Conditions have been checked by main thread. No exception handling needed.
		}
	}

}
