package eu.simstadt.nf4j.async;

import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Test;

import eu.simstadt.nf4j.FailedTransmissionException;
import eu.simstadt.nf4j.InvalidJobDescriptorException;
import eu.simstadt.nf4j.JobStatus;
import eu.simstadt.nf4j.async.AsyncImportJob;
import eu.simstadt.nf4j.async.HTTPConnection;
import eu.simstadt.nf4j.async.ImportJobDescription;
import eu.simstadt.nf4j.async.JobStatusEvent;
import eu.simstadt.nf4j.async.JobStatusListener;

/**
 * This class contains client oriented import job tests. It will send an import job and listens to
 * status updates. Every of the subsequent status' LOCAL, SENT, PENDING, RUNNING, FINISHED
 * have to be signaled to this test class. 
 *   
 * @author Marcel Bruse
 */
public class SuccessfulImportJob implements JobStatusListener {

    public AsyncImportJob job;
    
    @Test
    public void processJob() throws InterruptedException {
		ImportJobDescription desc = ImportJobDescription.getDefaultDescriptor();
		desc.setProduct("LBTEST");
		desc.setLeaf("GR");
		desc.setCityGMLFile(new File("SomeBuildings.gml"));
		
		HTTPConnection connector = new HTTPConnection("193.196.136.164");
		job = new AsyncImportJob(desc, connector);
		job.addJobStatusListener(this);
		
		try {
			job.send();
		} catch (InvalidJobDescriptorException ex) {
			ex.printStackTrace();
		} catch (FailedTransmissionException ex) {
			ex.printStackTrace();
		}
		
        // Wait for timeout, failure or that all tests pass
        long timeout = 1000 * 60 * 5l; // 5 minutes maximum
        long interval = 10000l;
        while (!job.hasFinished() && !job.hasFailed() && timeout > 0) {
        	Thread.sleep(interval);        	
        	timeout -= interval;
        }
    }

    @Override
    public void jobStatusChanged(JobStatusEvent event) {
        JobStatus status = (JobStatus) event.getSource();
        System.out.println(status);
        if (status == JobStatus.LOCAL) {
        	assertTrue(true);
        } else if (status == JobStatus.SENT) {
        	assertTrue(true);
        } else if (status == JobStatus.PENDING) {
        	assertTrue(true);
        } else if (status == JobStatus.RUNNING) {
        	assertTrue(true);
        } else if (status == JobStatus.FINISHED) {
        	assertTrue(true);
        }
    }

}